@@ -23,8 +23,16 @@ def add(collection, image):
     return ee.ImageCollection.fromImages(append)
 
 
+def allMasked(collection):
+    """ Get a mask which indicates pixels that are masked in all images (0) and
+    pixels that have a valid pixel in at least one image (1) """
+    masks = collection.map(lambda i: i.mask())
+    masksum = ee.Image(masks.sum())
+    return ee.Image(masksum.gt(0))
+
+
 def containsAllBands(collection, bands):
-    """ Filter a collection with images cotaining all bands specified in
+    """ Filter a collection with images containing all bands specified in
     parameter `bands` """
     bands = ee.List(bands)
     # add bands as metadata
@@ -137,27 +145,43 @@ def enumerateSimple(collection, name='ENUM'):
     return ee.ImageCollection(fc.copyProperties(source=collection))
 
 
-def fillWithLast(collection, unmask_first=False):
-    """ Fill masked values of each image pixel with the last available
-    value
+def fillWithLast(collection, reverse=False, date_property='system:time_start'):
+    """ Fill each masked pixels with the last available not masked pixel. If reverse, it goes backwards.
+    Images must contain a valid date (system:time_start property by default) """
+
+    def makereverse(col):
+        dates = ee.List(col.aggregate_array(date_property)).reverse()
+        col = ee.ImageCollection.fromImages(ee.List(dates.map(
+            lambda d: ee.Image(
+                col.filter(ee.Filter.eq(date_property, d)).first()))))
+        return col
+
+    if reverse:
+        collection = makereverse(collection)
 
-    :param collection: the collection that holds the images that will be filled
-    :type collection: ee.ImageCollection
-    :param unmask_first: unmask first image?
-    :type unmask_first: bool
-    :rtype: ee.ImageCollection
-    """
-    collector = ee.List([])
     def overcol(i, collect):
+        i = ee.Image(i)
         collect = ee.List(collect)
+
         def true():
             last = ee.Image(collect.get(-1))
-            mask = i.mask().Not()
-            return collect.add(ee.Image(i.unmask().where(mask, last)))
+            idate = ee.Number(i.get(date_property))
+            blended = i.unmask().where(i.mask().Not(), last)
+            return collect.add(blended)
+
         def false():
-            return collect.add(i.unmask() if unmask_first else i)
-        return ee.List(ee.Algorithms.If(collect.size(), true(), false()))
-    return ee.ImageCollection.fromImages(collection.iterate(overcol, collector))
+            return collect.add(i)
+
+        condition = collect.size()
+        return ee.List(ee.Algorithms.If(condition, true(), false()))
+
+    result = ee.ImageCollection.fromImages(
+        ee.List(collection.iterate(overcol, ee.List([]))))
+
+    if reverse:
+        result = makereverse(result)
+
+    return result
 
 
 def mergeGeometries(collection):
@@ -676,6 +700,101 @@ def linearFunctionProperty(collection, property, range_min=None,
     return collection
 
 
+def linearInterpolation(collection, date_property='system:time_start'):
+    def _addTime(collection):
+        def wrap(i):
+            sec = ee.Number(i.get(date_property))
+            isec = image_module.empty(sec, i.bandNames())
+            isec_suffix = image_module.addSuffix(isec, '_tmpTime')
+            m = i.mask()
+            isec_masked = isec.updateMask(m)
+            isec_masked_suffix = image_module.addSuffix(isec_masked,
+                                                        '_maskedTime')
+            return i.addBands(isec_suffix).addBands(isec_masked_suffix)
+
+        return collection.map(wrap)
+
+    # get the mask for the final result
+    finalmask = allMasked(collection)
+
+    # add time bands
+    collection = _addTime(collection)
+
+    filled = fillWithLast(collection, False, date_property=date_property)
+    filled_back = fillWithLast(collection, True, date_property=date_property)
+
+    condition = ee.Filter.equals(leftField='system:index',
+                                 rightField='system:index')
+
+    match1 = ee.Join.saveFirst('filled').apply(
+        primary=collection,
+        secondary=filled,
+        condition=condition
+    )
+
+    match2 = ee.Join.saveFirst('filled_back').apply(
+        primary=match1,
+        secondary=filled_back,
+        condition=condition
+    )
+
+    def wrap(image):
+        o = ee.Image(image)
+        bands = o.bandNames()
+
+        masked = o.mask().Not()
+        f = ee.Image(image.get('filled')).unmask()
+        fb = ee.Image(image.get('filled_back')).unmask()
+
+        # filters
+        filter0 = ee.Filter.stringContains('item', 'maskedTime')
+        filter1 = ee.Filter.stringContains('item', 'maskedTime').Not()
+        filter2 = ee.Filter.stringContains('item', 'tmpTime').Not()
+
+        # get all deltas (including delta x)
+        dy = ee.Image(fb.subtract(f)).unmask()
+        dx_bands = bands.filter(filter0)
+
+        # select only delta x for each band
+        dx = dy.select(dx_bands)
+
+        # get original bands
+        original_bands = bands.filter(filter1).filter(filter2)
+
+        # get delta for original bands
+        delta = dy.select(original_bands)
+
+        # now that we have delta x and delta for the original bands
+        # get the slope
+        slope = delta.divide(dx).unmask()
+
+        # filled original bands
+        fo = f.select(original_bands)
+
+        # filled back original bands
+        fob = fb.select(original_bands)
+
+        # original bands
+        oo = o.select(original_bands)
+
+        # masked original bands
+        mo = masked.select(original_bands)
+
+        t = o.select('.+_tmpTime').subtract(f.select('.+_maskedTime'))
+        fill = fo.add(slope.multiply(t)).unmask()
+        fill2 = fob.where(fill, fill)
+        fill3 = fo.where(fill2, fill2)
+        final = oo.unmask().where(mo, fill3)
+        final = image_module.deleteProperties(final)
+        final = final.select(original_bands) \
+            .copyProperties(o, exclude=['filled', 'filled_back']) \
+            .set(date_property, o.get(date_property)) \
+            .set('system:index', o.get('system:index'))
+        return ee.Image(final).updateMask(finalmask)
+
+    return ee.ImageCollection(match2.map(wrap))
+
+
 def gaussFunctionBand(collection, band, range_min=None, range_max=None,
                       mean=0, output_min=None, output_max=1, std=None,
                       stretch=1, name='gauss'):
@@ -870,4 +989,17 @@ def normalDistributionBand(collection, band, mean=None, std=None,
              .divide(istd.multiply(ee.Image.constant(2).multiply(ipi).sqrt()))
 
     return gaussFunctionBand(collection, band, mean=imean,
-                             output_max=imax, std=istd, name=name)
\ No newline at end of file
+                             output_max=imax, std=istd, name=name)
+
+
+def maskedSize(collection):
+    """ return an image with the percentage of masked pixels. 100% means all
+    pixels are masked """
+    mask = collection.map(lambda i: i.mask().Not())
+    def wrap(i):
+        onemore = i.add(1)
+        return onemore.divide(onemore)
+    total = mask.map(wrap)
+    masksum = mask.sum()
+    totalsum = total.sum()
+    return masksum.divide(totalsum).multiply(100).toInt()
